package seabedexplorer.submersible;

import java.util.Collection;
import seabedexplorer.data.Data;
import seabedexplorer.observer.Observer;
import seabedexplorer.submersible.equipment.locationtracking.GPS;
import seabedexplorer.submersible.equipment.locationtracking.Location;
import seabedexplorer.submersible.state.SubmersibleState;
import seabedexplorer.submersible.state.states.DockedState;

/**
 * Submersible
 * 
 * @author JLScott1999
 */
public abstract class Submersible implements Observer, Moveable
{
    
    /**
     * Data storage for the submarine
     */
    protected Collection<Data> dataCollection;
    
    /**
     * State of the submersible
     */
    private SubmersibleState currentState;
    
    private GPS gps;
    
    /**
     * Constructor creating a submersible with some type of data storage eg) ArrayList or LinkedList
     * @param dataCollection the data storage to use eg) ArrayList or LinkedList
     */
    public Submersible(Collection<Data> dataCollection)
    {
        this.dataCollection = dataCollection;
        gps = new GPS();
        currentState = new DockedState();
    }

    /**
     * Constructor creating a submersible without any data storage state or gps
     */
    public Submersible()
    {
        
    }
    
    /**
     * Get the submersible object
     * @return submersible object memory address
     */
    public abstract Submersible getSubmersible();
    
    /**
     * Get the submersible type
     * @return submersible type such as AUV, ROV, etc..
     */
    public abstract String getSubmersibleType();
    
    /**
     * Collect data using one of the equipment provided
     */
    public abstract void collectData();
    
    /**
     * Get the data stored on the submersible
     * @return the data stored on the submersible
     */
    public Collection<Data> getData() {
        return dataCollection;
    }
    
    /**
     * Add data to store on the submersible
     * @param data the data to store on the submersible
     */
    public void addData(Data data)
    {
        if (data != null)
        {
            dataCollection.add(data);
        }
    }
    
    /**
     * Clear the data stored on a submersible
     */
    public void clearData()
    {
        if (dataCollection != null)
        {
            dataCollection.removeAll(dataCollection);
        }
    }

    public void changeState(SubmersibleState state)
    {
        currentState = state;
        state.submersibleInstance(this);
        System.out.println("State Changed: " + currentState.toString());
    }

    public SubmersibleState getState()
    {
        return currentState;
    }

    public String getStateName()
    {
        return currentState.toString();
    }
    
    @Override
    public void move(Location loc)
    {
        currentState.move(loc);
    }
    
    @Override
    public void stopMove()
    {
        currentState.stopMove();
    }
    
    @Override
    public boolean canMove()
    {
        return currentState.canMove();
    }
    
    public boolean returnToVessel()
    {
        return currentState.returnToVessel();
    }
    
    public boolean collectWater()
    {
        return currentState.collectWater();
    }

    public boolean collectMaterial()
    {
        return currentState.collectMaterial();
    }

    public boolean collectMappingData()
    {
        return currentState.collectMappingData();
    }
    
    public GPS getGPS()
    {
        return gps;
    }
    
    public void setGPS(GPS gps)
    {
        this.gps = gps;
    }

    @Override
    public String toString()
    {
        String result = "";
        if (currentState != null)
        {
            result += "State: " + currentState.toString() + "\n";
        }
        if (dataCollection != null)
        {
            result += "Data:";
            for (Data d: dataCollection)
            {
                result += "\n    " + d.toString().replace("\n", "\n    ");
            }
        }
        return result + "\nEquipment:";
    }

}
