package seabedexplorer.util.submersible;

import java.security.InvalidParameterException;
import seabedexplorer.submersible.AUV;
import seabedexplorer.submersible.MS;
import seabedexplorer.submersible.ROV;
import seabedexplorer.submersible.Submersible;
import seabedexplorer.submersible.equipment.materialsampler.ChemicalScanner;
import seabedexplorer.submersible.equipment.materialsampler.HotFluidSampler;
import seabedexplorer.submersible.equipment.materialsampler.RockCoringDrill;
import seabedexplorer.submersible.equipment.topographicalmapper.Laser;
import seabedexplorer.submersible.equipment.topographicalmapper.Sonar;
import seabedexplorer.submersible.equipment.waterreadings.IndexOfRefraction;
import seabedexplorer.submersible.equipment.waterreadings.PH;
import seabedexplorer.submersible.equipment.waterreadings.Salinity;

/**
 * SubmersibleBuilder
 * 
 * @author JLScott1999
 */
public class SubmersibleBuilder
{
    
    private Submersible submersible;
    
    /**
     * Build a submersible easily
     * @param submersibleType Type of submersible to start the creation of.
     */
    public SubmersibleBuilder(SubmersibleType submersibleType)
    {
        System.out.println("--- Submarine Builder Start ---");
        switch (submersibleType)
        {
            case AUTOMATED_UNDERWATER_VEHICLE:
            {
                System.out.println("Creating a Automated Underwater Vehicle");
                submersible = new AUV();
                break;
            }
            case MANNED_SUBMERSIBLE:
            {
                System.out.println("Creating a Manned Submersible");
                submersible = new MS();
                break;
            }
            case REMOTE_OPERATED_VEHICLE:
            {
                System.out.println("Creating a Remote Operated Vehicle");
                submersible = new ROV();
                break;
            }
            default:
            {
                throw new InvalidParameterException("Use the SubmersibleType class to select which submersible to create");
            }
        }
    }
    
    /**
     * randomly add parts to a submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addRandom()
    {
        for (int i = 0; i < (int)(Math.random() * 10 + 1); i++)
        {
            switch ((int)(Math.random() * 8 + 1))
            {
                case 1:
                {
                    this.addLaser();
                    break;
                }
                case 2:
                {
                    this.addSonar();
                    break;
                }
                case 3:
                {
                    this.addIndexOfRefraction();
                    break;
                }
                case 4:
                {
                    this.addPH();
                    break;
                }
                case 5:
                {
                    this.addSalinity();
                    break;
                }
                case 6:
                {
                    this.addChemicalScanner();
                    break;
                }
                 case 7:
                {
                    this.addHotFluidSampler();
                    break;
                }
                case 8:
                {
                    this.addRockCoringDrill();
                    break;
                }
            }
        }
        return this;
    }
    
    /**
     * Add laser-illuminated photography to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addLaser()
    {
        System.out.println("Adding Laser-illuminated photography");
        submersible = new Laser(submersible, (double)(Math.random() * 30 + 10) / 10);
        return this;
    }
    
    /**
     * Add sonar to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addSonar()
    {
        System.out.println("Adding Sonar");
        submersible = new Sonar(submersible);
        return this;
    }
    
    /**
     * Add index of refraction reader to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addIndexOfRefraction()
    {
        System.out.println("Adding Index of Refraction reader");
        submersible = new IndexOfRefraction(submersible, (Math.random() * 88 + 1) + Math.random());
        return this;
    }
    
    /**
     * Add ph reader to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addPH()
    {
        System.out.println("Adding PH reader");
        submersible = new PH(submersible);
        return this;
    }
    
    /**
     * Add salinity reader to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addSalinity()
    {
        System.out.println("Adding Salinity reader");
        submersible = new Salinity(submersible);
        return this;
    }
    
    /**
     * Add chemical scanner to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addChemicalScanner()
    {
        System.out.println("Adding Chemical Scanner");
        submersible = new ChemicalScanner(submersible, (Math.random() * 150 + 10));
        return this;
    }
    
    /**
     * Add hot fluid sampler to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addHotFluidSampler()
    {
        System.out.println("Adding Fluid Sampler");
        submersible = new HotFluidSampler(submersible, (Math.random() * 150 + 10));
        return this;
    }
    
    /**
     * Add rock coring drill to the submersible
     * @return The builder instance so it more can be added to it
     */
    public SubmersibleBuilder addRockCoringDrill()
    {
        System.out.println("Adding RockCoring Drill");
        submersible = new RockCoringDrill(submersible, (Math.random() * 150 + 10));
        return this;
    }
    
    /**
     * Adds all equipment to the submarine
     */
    public void addAll()
    {
        addLaser();
        addSonar();
        addIndexOfRefraction();
        addPH();
        addSalinity();
        addChemicalScanner();
        addHotFluidSampler();
        addRockCoringDrill();
    }
    
    /**
     * finish the submarine build
     * @return Submersible The submarine class with all the parts added to it
     */
    public Submersible build()
    {
        System.out.println("--- Submarine Builder Finish ---");
        return submersible;
    }
    
}
